<?php
/**
 * Functions required to start a Gollem session.
 *
 * $Horde: gollem/lib/Session.php,v 1.11 2005/01/03 14:34:54 jan Exp $
 *
 * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Max Kalika <max@horde.org>
 * Copyright 2004-2005 Michael Slusarz <slusarz@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Max Kalika <max@horde.org>
 * @author  Michael Slusarz <slusarz@horde.org>
 * @version $Revision: 1.11 $
 * @since   Gollem 1.0
 * @package Gollem
 */
class Gollem_Session {

    /**
     * Take information posted from a login attempt and try setting up
     * an initial Gollem session. Handle Horde authentication, if
     * required, and only do enough work to see if the user can log
     * in. This function should only be called once, when the user first logs
     * into Gollem.
     *
     * Creates the $gollem session variable with the following entries:
     * 'attributes'  --  See config/backends.php
     * 'autologin'   --  TODO
     * 'backend_key' --  TODO
     * 'dir'         --  The current directory
     * 'driver'      --  See config/backends.php
     * 'home'        --  The user's home directory
     * 'hordeauth'   --  See config/backends.php
     * 'hostspec'    --  TODO
     * 'label'       --  The label to use
     * 'name'        --  See config/backends.php
     * 'params'      --  See config/backends.php
     * 'preferred'   --  See config/backends.php
     * 'root'        --  The root directory
     * 'selectlist'  --  Stores file selections from the API call
     *
     * @access public
     *
     * @param string $key            The backend key to initialize.
     * @param optional string $user  The username to use for authentication.
     * @param optional string $pass  The password to use for authentication.
     * @param optional array $args   Any additional parameters the backend
     *                               needs.
     *
     * @return boolean  True on success, false on failure.
     */
    function createSession($key, $user = null, $pass = null, $args = array())
    {
        global $conf;

        /* Make sure we have a key and that it is valid. */
        if (empty($key) || (substr($key, 0, 1) == '_')) {
            return false;
        }

        /* We might need to override some of the defaults with
         * environment-wide settings. Do NOT use the global $backends
         * variable as it may not exist. */
        require GOLLEM_BASE . '/config/backends.php';
        if (empty($backends[$key])) {
            $entry = sprintf('Invalid server key from client [%s]', $_SERVER['REMOTE_ADDR']);
            Horde::logMessage($entry, __FILE__, __LINE__, PEAR_LOG_INFO);
            return false;
        }

        /* Create gollem session object. */
        $_SESSION['gollem'] = array();
        $_SESSION['gollem'] = $backends[$key];
        $ptr = &$_SESSION['gollem'];

        $ptr['backend_key'] = $key;
        $ptr['params'] = array_merge($ptr['params'], $args);

        /* Only allow server/port changing for FTP backends. */
        if (($conf['backend']['backend_list'] == 'none') ||
            ($ptr['driver'] == 'ftp')) {

            if ($conf['backend']['change_server'] && !empty($args['server'])) {
                $ptr['driver'] = 'ftp';
                $ptr['params']['hostspec'] = $args['server'];
            }

            if ($conf['backend']['change_port'] && !empty($args['port'])) {
                $ptr['driver'] = 'ftp';
                $ptr['params']['hostspec'] = $args['port'];
            }
        }

        /* Set username now. Don't set the current username if the backend
         * already has a username defined. */
        if (empty($ptr['params']['username'])) {
            $ptr['params']['username'] = (is_null($user)) ? Auth::getBareAuth() : $user;
        }

        /* Set password now. The password should always be encrypted within
         * the session. */
        if (!empty($ptr['params']['password'])) {
            $pass = $ptr['params']['password'];
        }
        if (!is_null($pass)) {
            $ptr['params']['password'] = Secret::write(Secret::getKey('gollem'), $pass);
        }

        /* Try to authenticate with the given information. */
        $auth_gollem = &Auth::singleton(array('gollem', 'gollem'));
        if ($auth_gollem->authenticate(null, null, true) !== true) {
            return false;
        }

        // Make sure we have a 'home' parameter.
        if (empty($ptr['home'])) {
            $ptr['home'] = (!empty($ptr['params']['home'])) ? $ptr['params']['home'] : $GLOBALS['vfs']->getCurrentDirectory();
        }
        $ptr['dir'] = $ptr['home'];

        // Make sure we have a 'root' parameter.
        if (empty($ptr['root'])) {
            $ptr['root'] = $ptr['home'];
        }

        // Verify that home is below root.
        if (!Gollem::verifyDir($ptr['home'])) {
            Horde::logMessage(PEAR::raiseError('Home directory not below root.'), __FILE__, __LINE__, PEAR_LOG_ERR);
            return false;
        }

        /* Does this driver support autologin? */
        $ptr['autologin'] = Gollem::canAutoLogin(true);

        /* Initialize the 'selectlist' cache. */
        $ptr['selectlist'] = array();

        return true;
    }

}
